/*
 * This file is part of the Molecfit Pipeline
 * Copyright (C) 2001-2019 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef MOLECFIT_H
#define MOLECFIT_H

/*----------------------------------------------------------------------------*/
/**
 *                              Includes
 */
/*----------------------------------------------------------------------------*/

#include <cpl.h>

#include <telluriccorr.h>

#include "molecfit_dfs.h"
#include "molecfit_config.h"
#include "molecfit_data.h"

/*----------------------------------------------------------------------------*/
/**
 *                              Defines
 */
/*----------------------------------------------------------------------------*/

/* MOLECFIT RECIPES */
#define MOLECFIT_MODEL       "molecfit_model"
#define MOLECFIT_CALCTRANS   "molecfit_calctrans"
#define MOLECFIT_CORRECT     "molecfit_correct"

/*----------------------------------------------------------------------------*/
/**
 *                 Typedefs: Structs and enum types
 */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
 *                              Functions prototypes
 */
/*----------------------------------------------------------------------------*/

/* Check inputs in the recipe */
cpl_error_code molecfit_check_and_set_groups(
  cpl_frameset *frameset);

/* Load kernel tag */
molecfit_fits * molecfit_load_kernel_tag(
    cpl_frameset             *frameset,
    const char               *tag_kernel,
    cpl_boolean              use_input_kernel,
    mf_parameters_config     *parameters,
    cpl_boolean               verbose);

/* Convert input data to internal spectrum */
cpl_error_code molecfit_data_convert_to_table(
    molecfit_fits            *data,
    cpl_boolean              chip_extensions,
    cpl_boolean              use_only_pri_ext,
    int                      dflux_extension_data,
    int                      mask_extension_data,
    const char               *column_lambda,
    const char               *column_flux,
    const char               *column_dflux,
    const char               *column_mask);

/* Load ranges in a input FITS BINTABLE with one only extension */
cpl_table * molecfit_load_unique_table(
    const cpl_frameset       *frameset,
    const char               *tag);

/* Load table in a input FITS BINTABLE of the extension 'ext' */
cpl_table * molecfit_load_table(
    const cpl_frame          *frame,
    const cpl_size           ext);

/* Save to disk a new *.fits output file */
cpl_error_code molecfit_save(
    cpl_frameset             *all_frameset,
    cpl_frameset             *used_frameset,
    const cpl_parameterlist  *parlist,
    const char               *recipe,
    cpl_propertylist         *list,
    const char               *tag,
    const char               *filename);

/* Save to disk a cpl_table output for one molecfit execution */
cpl_error_code molecfit_save_mf_results(
    cpl_propertylist         *header_data,
    const char               *name,
    cpl_boolean              addFitsExtension,
    cpl_matrix               *matrix,
    cpl_table                *table,
    cpl_vector               *vec);


#endif /* MOLECFIT_H */
