/*
 * This file is part of the Molecfit Pipeline
 * Copyright (C) 2001-2019 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef MOLECFIT_CONFIG_H
#define MOLECFIT_CONFIG_H

/*----------------------------------------------------------------------------*/
/**
 *                              Includes
 */
/*----------------------------------------------------------------------------*/

#include <cpl.h>

#include <telluriccorr.h>
#include <molecfit_fits.h>
#include <molecfit.h>
/*#include <molecfit.h>*/

CPL_BEGIN_DECLS

/*----------------------------------------------------------------------------*/
/**
 *                              Defines
 */
/*----------------------------------------------------------------------------*/

/* COLUMN MAPPING PARAMETERS */
#define MOLECFIT_MAPPING_KERNEL_EXT                        "KERNEL_LIBRARY_EXT"
#define MOLECFIT_MAPPING_ATMOSPHERIC_EXT                   "ATM_PARAMETERS_EXT"
#define MOLECFIT_MAPPING_CONVOLVE_EXT                      "LBLRTM_RESULTS_EXT"
#define MOLECFIT_MAPPING_CORRECT_EXT                       "TELLURIC_CORR_EXT"
#define MOLECFIT_FIT_MODEL_EXT		                       "TELLURIC_DATA_EXT"



/* RECIPE PARAMETERS */
#define MOLECIFT_TELLURICCORR_PARAMETER_DEFAULT            "TELLURICCORR_PARAMETER_DEFAULT"


#define MOLECFIT_PARAMETER_USE_ONLY_INPUT_PRI              "USE_ONLY_INPUT_PRIMARY_DATA"
#define MOLECFIT_PARAMETER_USE_ONLY_INPUT_PRI_DESC         "Value=TRUE implies that only the fits primary contains the input science flux data.\n"\
                                                           " Value=FALSE implies that the fits extensions also contains input science flux data."
#define MOLECFIT_PARAMETER_USE_ONLY_INPUT_PRI_INIT         CPL_FALSE

#define MOLECFIT_PARAMETER_DFLUX_EXTENSION_DATA            "USE_DATA_EXTENSION_AS_DFLUX"
#define MOLECFIT_PARAMETER_DFLUX_EXTENSION_DATA_DESC       "Only valid if USE_ONLY_INPUT_PRIMARY_DATA=TRUE. The fits extension index that contains the\n"\
                                                           " errors of the science flux data (DFLUX). A value of 0 implies that there is no DFLUX"
#define MOLECFIT_PARAMETER_DFLUX_EXTENSION_DATA_INIT       0

#define MOLECFIT_PARAMETER_MASK_EXTENSION_DATA             "USE_DATA_EXTENSION_AS_MASK"
#define MOLECFIT_PARAMETER_MASK_EXTENSION_DATA_DESC        "Only valid if USE_ONLY_INPUT_PRIMARY_DATA=TRUE. The fits extension index that contains the\n"\
                                                           " mask associated with the science flux data. A value of 0 implies that there is no mask data." 
#define MOLECFIT_PARAMETER_MASK_EXTENSION_DATA_INIT        0

#define MOLECFIT_PARAMETER_USE_INPUT_KERNEL                "USE_INPUT_KERNEL"
#define MOLECFIT_PARAMETER_USE_INPUT_KERNEL_DESC           "If TRUE, use the kernel library if it is provided."
#define MOLECFIT_PARAMETER_USE_INPUT_KERNEL_INIT           CPL_TRUE

#define MOLECFIT_PARAMETER_SUPPRESS_EXTENSION              "SUPPRESS_EXTENSION"
#define MOLECFIT_PARAMETER_SUPPRESS_EXTENSION_DESC         "Suppress arbitrary filename extension : TRUE (apply) or FALSE (don't apply)"
#define MOLECFIT_PARAMETER_SUPPRESS_EXTENSION_INIT         CPL_FALSE

#define MOLECFIT_PARAMETER_CHIP_EXTENSIONS                 "CHIP_EXTENSIONS"
#define MOLECFIT_PARAMETER_CHIP_EXTENSIONS_DESC            "Flag that determines if image extensions are to be treated as independent\n"\
                                                           " science data to be fitted for independently or as CHIP specific subranges \n"\
                                                           " of a single observation to be fitted for as a single combined spectrum.\n"\
                                                           " Value = TRUE implies to treat as CHIPS to be combined. Value = FALSE implies\n"\
                                                           " to treat as independent. [FALSE]\n"
#define MOLECFIT_PARAMETER_CHIP_EXTENSIONS_INIT            CPL_FALSE

#define MOLECFIT_PARAMETER_WLC_REF                 		   "WLC_REF"
#define MOLECFIT_PARAMETER_WLC_REF_DESC                    "Indicates that the reference for the wavelength calibration :\n"\
                                                      	   " - If it is set to 'DATA',  is the input  data.\n"\
														   " - If it is set to 'MODEL', is the output model.\n"
#define MOLECFIT_PARAMETER_WLC_REF_INIT            		   "DATA"


/* Molecules table */
#define MOLECFIT_PARAMETER_LIST                            MF_COL_LIST_MOLECULES
#define MOLECFIT_PARAMETER_LIST_DESC                       "List of molecules to be included in the model. Represented as a comma separated\n"\
                                                           " string of molecule names, e.g. \"H2O,CO2,O3\".\n"\
                                                           " If set to NULL, the input TAG[MOLECULES] FITS BINTABLE values have to be provided\n"\
                                                           " where the FITS BINTABLE specified contains the three columns:\n"\
                                                           " LIST_MOLEC; FIT_MOLEC; and REL_COL."
#define MOLECFIT_PARAMETER_LIST_INIT                       MF_PARAMETERS_NULL

#define MOLECFIT_PARAMETER_FIT                             MF_COL_FIT_MOLECULES
#define MOLECFIT_PARAMETER_FIT_DESC                        "List of flags that specify which of the listed molecules are to be fitted for.\n"\
                                                           " Flag=1 implies yes. Flag=0 implies no. Represented as a string of comma separated\n"\
                                                           " integers in the same order as the listed molecules. For example: if LIST_MOLEC=\"H2O,CO2,O3\", then \n"\
                                                           " FIT_MOLEC=\"1,0,1\" implies that only H2O and O3 should be fitted for.\n"\
                                                           " If set to NULL, the input TAG[MOLECULES] FITS BINTABLE values have to be provided where the FITS \n"\
                                                           " BINTABLE specified contains the three columns: LIST_MOLEC; FIT_MOLEC; and REL_COL."
#define MOLECFIT_PARAMETER_FIT_INIT                        MF_PARAMETERS_NULL

#define MOLECFIT_PARAMETER_RELATIVE_VALUE                  MF_COL_REL_COL
#define MOLECFIT_PARAMETER_RELATIVE_VALUE_DESC             "List of the intial values of fitting of the molecular columns expressed relatively to the input\n"\
                                                           "  ATM profile columns. Represented as a comma separated list of doubles in the same order as the \n"\
                                                           " listed molecules. For example, if LIST_MOLEC=\"H2O,CO2,O3\", then REL_COL=\"1.0,1.2,0.8\"\n"\
                                                           " implies that H2O, CO2 and O3 have initial relative values of 1.0, 1.2 and 0.8 respectively.\n"\
                                                           " If set to NULL, the input TAG[MOLECULES] FITS BINTABLE values have to be provided where the FITS \n"\
                                                           " BINTABLE specified contains the three columns: LIST_MOLEC; FIT_MOLEC; and REL_COL."
#define MOLECFIT_PARAMETER_RELATIVE_VALUE_INIT             MF_PARAMETERS_NULL


/* Range tables */
#define MOLECFIT_PARAMETER_WAVE_RANGE_INCLUDE              MOLECFIT_WAVE_INCLUDE
#define MOLECFIT_PARAMETER_WAVE_RANGE_INCLUDE_DESC         "Wavelength ranges to be included. Represented as a string of comma separated doubles in pairs \n"\
                                                           " specifying the start and end wavelengths of a range. The wavelength units are always in microns.\n"\
                                                           " For example a KMOS sample data in the range of 1.11um to 1.67um may have\n"\
                                                           " WAVE_INCLUDE=\"1.773,1.78633,1.79098,1.80434,1.187691,1.189937\" to represent three inclusion regions:\n"\
                                                           " [1.773,1.78633], [1.79098,1.80434] and [1.187691,1.189937].\n"\
                                                           " If set to NULL, molecfit will check if the TAG[WAVE_INCLUDE] FITS BINTABLE values is provided where\n"\
                                                           " the FITS BINTABLE specified has the two columns: LOWER_LIMIT; and UPPER_LIMIT."
#define MOLECFIT_PARAMETER_WAVE_RANGE_INCLUDE_INIT         MF_PARAMETERS_NULL

#define MOLECFIT_PARAMETER_WAVE_RANGE_EXCLUDE              MOLECFIT_WAVE_EXCLUDE
#define MOLECFIT_PARAMETER_WAVE_RANGE_EXCLUDE_DESC         "Wavelength ranges to be excluded. Represented as a string of comma separated doubles in pairs \n"\
                                                           " specifying the start and end wavelengths of a range. The wavelength units are always in microns.\n"\
                                                           " as the input science data. For example a KMOS sample data in the range of 1.11um to 1.67um may have\n"\
                                                           " WAVE_EXCLUDE=\"1.773,1.78633,1.79098,1.80434,1.187691,1.189937\" to represent three exclusion regions:\n"\
                                                           " [1.773,1.78633], [1.79098,1.80434] and [1.187691,1.189937].\n"\
                                                           " If set to NULL, molecfit will check if the TAG[WAVE_EXCLUDE] FITS BINTABLE values is provided where\n"\
                                                           " the FITS BINTABLE specified has the two columns: LOWER_LIMIT; and UPPER_LIMIT."
#define MOLECFIT_PARAMETER_WAVE_RANGE_EXCLUDE_INIT         MF_PARAMETERS_NULL

#define MOLECFIT_PARAMETER_PIXEL_RANGE_EXCLUDE             MOLECFIT_PIXEL_EXCLUDE
#define MOLECFIT_PARAMETER_PIXEL_RANGE_EXCLUDE_DESC        "Pixel ranges to be excluded. Represented as a string of comma separated integers in pairs specifying the\n"\
                                                           " start and end pixel of a range. For example: PIXEL_EXCLUDE=\"54,128,512,514,1020,1024\" represents three\n"\
                                                           " exclusion regions: [54,128], [512,514] and [1020,1024].\n"\
                                                           " If set to NULL, molecfit will check if the TAG[PIXEL_EXCLUDE] FITS BINTABLE values is provided where the\n"\
                                                           " FITS BINTABLE specified has the two columns: LOWER_LIMIT; and UPPER_LIMIT."
#define MOLECFIT_PARAMETER_PIXEL_RANGE_EXCLUDE_INIT        MF_PARAMETERS_NULL


/*** MAPPING ***/
#define MOLECFIT_MAPPING_DEFAULT_PRIMARY                   MF_PARAMETERS_NULL


/* Mapping MODEL STD_MODEL/SCIENCE - KERNEL_EXT table */
#define MOLECFIT_PARAMETER_MODEL_MAPPING_KERNEL            MOLECFIT_MODEL_MAPPING_KERNEL
#define MOLECFIT_PARAMETER_MODEL_MAPPING_KERNEL_DESC       "Mapping '"MOLECFIT_STD_MODEL"/"MOLECFIT_SCIENCE"' - '"MOLECFIT_MODEL_KERNEL_LIBRARY"' [string with ext_number comma separated (int)] :\n"\
                                                           " If set to NULL, check if the TAG["MOLECFIT_MODEL_MAPPING_KERNEL"] FITS BINTABLE values is provided.\n"\
                                                           " The FITS BINTABLE have to one column ["MOLECFIT_MAPPING_KERNEL_EXT"]"
#define MOLECFIT_PARAMETER_MODEL_MAPPING_KERNEL_INIT       MOLECFIT_MAPPING_DEFAULT_PRIMARY

/* Mapping CALCTRANS SCIENCE - KERNEL_EXT table */
#define MOLECFIT_PARAMETER_CALCTRANS_MAPPING_KERNEL        MOLECFIT_CALCTRANS_MAPPING_KERNEL
#define MOLECFIT_PARAMETER_CALCTRANS_MAPPING_KERNEL_DESC   "Mapping '"MOLECFIT_SCIENCE"' - '"MOLECFIT_CALCTRANS_KERNEL_LIBRARY"' [string with ext_number comma separated (int)] :\n"\
                                                           " If set to NULL, check if the TAG["MOLECFIT_CALCTRANS_MAPPING_KERNEL"] FITS BINTABLE values is provided.\n"\
                                                           " The FITS BINTABLE have to one column ["MOLECFIT_MAPPING_KERNEL_EXT"]"
#define MOLECFIT_PARAMETER_CALCTRANS_MAPPING_KERNEL_INIT   MOLECFIT_MAPPING_DEFAULT_PRIMARY


/* Mapping CALCTRANS SCIENCE_EXT - ATM_PARAMETERS_EXT input cpl_table */
#define MOLECFIT_PARAMETER_MAPPING_ATMOSPHERIC             MOLECFIT_MAPPING_ATMOSPHERIC
#define MOLECFIT_PARAMETER_MAPPING_ATMOSPHERIC_DESC        "Mapping '"MOLECFIT_SCIENCE"' - '"MOLECFIT_ATM_PARAMETERS"' input [string with ext_number comma separated (int)] :\n"\
                                                           " If set to NULL, check if the TAG["MOLECFIT_MAPPING_ATMOSPHERIC"] FITS BINTABLE value is provided.\n"\
                                                           " The FITS BINTABLE have to one column ["MOLECFIT_MAPPING_ATMOSPHERIC_EXT"]"
#define MOLECFIT_PARAMETER_MAPPING_ATMOSPHERIC_INIT        MOLECFIT_MAPPING_DEFAULT_PRIMARY


/* Mapping LBLRTM_RESULTS_EXT - TELLURIC_CORR output cpl_table */
#define MOLECFIT_PARAMETER_MAPPING_CONVOLVE                MOLECFIT_MAPPING_CONVOLVE
#define MOLECFIT_PARAMETER_MAPPING_CONVOLVE_DESC           "Mapping '"MOLECFIT_LBLRTM_RESULTS"' - '"MOLECFIT_TELLURIC_CORR"' output [string with ext_number comma separated (int)] :\n"\
                                                           " If set to NULL, check if the TAG["MOLECFIT_MAPPING_CONVOLVE"] FITS BINTABLE value is provided.\n"\
                                                           " The FITS BINTABLE have to one column ["MOLECFIT_MAPPING_CONVOLVE_EXT"]"
#define MOLECFIT_PARAMETER_MAPPING_CONVOLVE_INIT           MOLECFIT_MAPPING_DEFAULT_PRIMARY


/* Mapping CORRECT SCIENCE_EXT - TELLURIC_CORR_EXT table */
#define MOLECFIT_PARAMETER_MAPPING_CORRECT                 MOLECFIT_MAPPING_CORRECT
#define MOLECFIT_PARAMETER_MAPPING_CORRECT_DESC            "Mapping '"MOLECFIT_SCIENCE"' - '"MOLECFIT_TELLURIC_CORR"' [string with ext_number comma separated (int)] :\n"\
                                                           " If set to NULL, check if the TAG["MOLECFIT_MAPPING_CORRECT"] FITS BINTABLE value is provided.\n"\
                                                           " The FITS BINTABLE have to one column ["MOLECFIT_MAPPING_CORRECT_EXT"]"
#define MOLECFIT_PARAMETER_MAPPING_CORRECT_INIT            MOLECFIT_MAPPING_DEFAULT_PRIMARY

/* Mapping CORRECT MODEL- TELLURIC_DATA table */
#define MOLECFIT_PARAMETER_FIT_MODEL                 	   MOLECFIT_FIT_MODEL
#define MOLECFIT_PARAMETER_FIT_MODEL_DESC            	   "Mapping '"MOLECFIT_SCIENCE"' - '"MOLECFIT_FIT_MODEL"' [string with ext_number comma separated (int)] :\n"\
                                                           " If set to NULL, check if the TAG["MOLECFIT_FIT_MODEL"] FITS BINTABLE value is provided.\n"\
#define MOLECFIT_PARAMETER_FIT_MODEL_INIT            	   MOLECFIT_MAPPING_DEFAULT_PRIMARY

#define MOLECFIT_PARAMETER_CALCTRANS_SCALE_PWV             "SCALE_PWV"
#define MOLECFIT_PARAMETER_CALCTRANS_SCALE_PWV_DESC        "Value read from science file to scale the precipitable water vapor (PWV) of the telluric spectra.\n"\
                                                           " If set to 'auto', the mean value of TEL AMBI IWV START/END is read from the header of the input science file.\n"\
                                                           " If set to a user-provided numerical value, use this as the scale factor.\n"\
                                                           " If set to a FITS header keyword, read in the scale factor from the header of the input science file using this keyword.\n"\
                                                           " If set to 'none', perform no scaling.\n"\

#define MOLECFIT_PARAMETER_CALCTRANS_SCALE_PWV_INIT        "none"

#define MOLECFIT_PARAMETER_CALCTRANS_PWV_HDR_MJD           "HDR_MJD"
#define MOLECFIT_PARAMETER_CALCTRANS_PWV_HDR_MJD_DESC      "FITS header keyword to read the Modified Julian Date at the START of the exposure from telluric and science header. The telluric header was copied to BEST_FIT_PARAMETERS by molecfit_model and read from there."
#define MOLECFIT_PARAMETER_CALCTRANS_PWV_HDR_MJD_INIT      "MJD-OBS"

#define MOLECFIT_PARAMETER_CALCTRANS_PWV_HDR_EXP           "HDR_EXP"
#define MOLECFIT_PARAMETER_CALCTRANS_PWV_HDR_EXP_DESC      "FITS header keyword to read the exposure time from telluric and science header. The telluric header was copied to BEST_FIT_PARAMETERS by molecfit_model and read from there."
#define MOLECFIT_PARAMETER_CALCTRANS_PWV_HDR_EXP_INIT      "ESO OBS EXECTIME"

#define MOLECFIT_PARAMETER_CALCTRANS_PWV_HDR_AIR1           "HDR_AIR1"
#define MOLECFIT_PARAMETER_CALCTRANS_PWV_HDR_AIR1_DESC      "FITS header keyword to read the airmass at the start of the exposure from the telluric and science header. The telluric header was copied to BEST_FIT_PARAMETERS by molecfit_model and read from there."
#define MOLECFIT_PARAMETER_CALCTRANS_PWV_HDR_AIR1_INIT      "ESO TEL AIRM START"

#define MOLECFIT_PARAMETER_CALCTRANS_PWV_HDR_AIR2           "HDR_AIR2"
#define MOLECFIT_PARAMETER_CALCTRANS_PWV_HDR_AIR2_DESC      "FITS header keyword to read the airmass at the end of the exposure from the telluric and science header. The telluric header was copied to BEST_FIT_PARAMETERS by molecfit_model and read from there."
#define MOLECFIT_PARAMETER_CALCTRANS_PWV_HDR_AIR2_INIT      "ESO TEL AIRM END"

#define MOLECFIT_PARAMETER_CALCTRANS_SG_WINDOW_LENGTH      "SGWL"
#define MOLECFIT_PARAMETER_CALCTRANS_SG_WINDOW_LENGTH_DESC "Savitzky-Golay filter smoothing window length in pixels"
#define MOLECFIT_PARAMETER_CALCTRANS_SG_WINDOW_LENGTH_INIT 15

#define MOLECFIT_PARAMETER_CALCTRANS_SG_WINDOW_LENGTH_AS_MAX      "SGWL_ASMAX"
#define MOLECFIT_PARAMETER_CALCTRANS_SG_WINDOW_LENGTH_AS_MAX_DESC "Treat the Savitzky-Golay filter smoothing window length as a maximum length"
#define MOLECFIT_PARAMETER_CALCTRANS_SG_WINDOW_LENGTH_AS_MAX_INIT CPL_FALSE


/*----------------------------------------------------------------------------*/
/**
 *                 Typedefs: Structs and enum types
 */
/*----------------------------------------------------------------------------*/


/* Enumeration for the different types of Molecfit string list parameter types */
typedef enum {
  SLIST,                  /* String list of strings                */
  DLIST,                  /* String list of doubles                */
  BLIST,                  /* String list of Booleans               */
  NLIST,                  /* String list of positive integers > 0  */
  N0LIST,                 /* String list of positive integers >=0  */
  DRANGE,                 /* Range of doubles                      */
  NRANGE,                 /* Range of positive integers > 0        */
  N0RANGE,                 /* Range of positive integers >=0        */
  BLIST_OLD               /* B LIST that could be a single value "FALSE" or "TRUE" */
} molecfit_strlst_type;

enum MF_STRLST_TYPE {
                     MF_STRLST_NULL,
                     MF_STRLST_STRING,
                     MF_STRLST_DOUBLE,
                     MF_STRLST_DRANGE,
                     MF_STRLST_INT,
                     MF_STRLST_BOOL
};


/*----------------------------------------------------------------------------*/
/**
 *                              Functions prototypes
 */
/*----------------------------------------------------------------------------*/

/* Insert parameter in the input parameterlist */
cpl_error_code molecfit_config_fill_parameter(
    const char               *recipe,
    cpl_parameterlist        *plist,
    const char               *param,
    cpl_boolean              range,
    const void               *rMin,
    const void               *rMax,
    cpl_type                 type,
    enum MF_STRLST_TYPE      strlst_type,
    const void               *value,
    const char               *description,
    cpl_boolean              mf_conf);

/* Add molecfit config parameters to the recipe parameters */
cpl_error_code molecfit_config_fill_parameterlist(
    const char               *recipe,
    cpl_parameterlist        *self);

/* Get molecfit config parameters from the input recipe parameter list */
mf_configuration * molecfit_config_get_parameters(
    const cpl_parameterlist  *list,
    const cpl_propertylist   *data_primary_header,
    cpl_propertylist         *pl);


/* Parse output header and add to the mf_parameters_config and the header propertylist */
mf_configuration * molecfit_config_load_header_parameters(
    const cpl_propertylist   *header,
    const cpl_propertylist   *data_header,
    cpl_propertylist         *pl);

/* Cleanup any memory allocations that were created by calling
     molecfit_config_load_header_parameters */
void molecfit_config_delete_configuration(mf_configuration* cfg) ;


/* Build the molecules Molecfit cpl_table */
cpl_table * molecfit_config_table_molecules(
    const char               *list_molec_str,
    const char               *fit_molec_str,
    const char               *rel_col_str);

/* Build the wave/pixel range Molecfit cpl_table */
cpl_table * molecfit_config_table_ranges(
    const char              *range_str,
    cpl_type                type);

/* Build mapping cpl_table */
cpl_table * molecfit_config_table_mapping(
    const char              *mapping_str,
    const char              *column_name);

/* Check that input string parameter is a valid list*/
cpl_error_code  molecfit_config_strlstchk(
    const cpl_parameterlist* parlist,
    const char*              parname,
    molecfit_strlst_type strlst_type);

/* Check that keyword exists*/
cpl_error_code mf_config_check_keyword(
    const molecfit_fits* fits_data,
    const cpl_parameterlist* parlist,
    const char* keyword_par);

/* Check that input string parameter is a valid option*/    
cpl_error_code  molecfit_config_stroptchk(
    const cpl_parameterlist* parlist,
    const char*              parname,
    const char*              options);
    
cpl_error_code mf_config_chk_gdas_prof(const char* file);    
/* Stringlist API*/
cpl_boolean  mf_strlst_is_null       (const char* strlst);
int          mf_strlst_get_size      (const char* strlst);
cpl_boolean  mf_strlst_get_str       (const char* strlst, int idx, char* elem_str);
double       mf_strlst_get_double    (const char* strlst, int idx);
float        mf_strlst_get_float     (const char* strlst, int idx);
int          mf_strlst_get_int       (const char* strlst, int idx);
cpl_boolean  mf_strlst_get_boolean   (const char* strlst, int idx);
cpl_boolean  mf_strlst_is_valid      (const char* strlst, enum MF_STRLST_TYPE strlst_type);
cpl_boolean  mf_strlst_is_valid_list_of_types 
                                     (const char* strlst, cpl_type data_type) ;
cpl_boolean  molecfit_config_str_isin_strlst(
    const char* str,
    const char* options_strlst);

/* Checks for specified named columns in fits data extensions */
cpl_error_code mf_config_chk_column(const molecfit_fits* data,const char* col_name);

/* Checks for specified keyword name in fits header */
cpl_error_code mf_config_chk_keyword(
    const molecfit_fits* fits_data,
    const char* keyword);

/* Checks that specified refernce atmosphere fits file has correct molecules */
cpl_error_code mf_config_chk_ref_atmos(
    const char* dirname,
    const char* filename,
    const cpl_table* molecules);

/* Checks that specified lnfl line database exists */
cpl_error_code mf_config_chk_lnfl_line_db(
    const char* dirname, 
    const char* db);

/* Checks that mappings are valid*/
cpl_error_code  molecfit_config_mappingchk(
   const cpl_frameset* frameset,
   const cpl_parameterlist* parlist,
   const char*  library,
   const char*  lib_map,
   const char*  ext_column,
   const char*  parname,
   const char*  target_tag);
    
/* Checks the wvelength strings are copatible */
cpl_error_code mf_config_chk_wlc_strs(
    const char* wlc_n_str, 
    const char* wlc_c_str,
    int nchips);
    
/* Checks that molecule string lists are self compatible */
cpl_error_code molecfit_config_molecule_strs_chk(const char* str1,const char* str4,const char* str3);

/* Check the science data file type is a table and not an image*/
cpl_boolean  molecfit_config_chk_science_is_table(cpl_frame *frame);

CPL_END_DECLS


#endif /* MOLECFIT_CONFIG_H */
